/**
 * Java implementation of DKIM/DomainKeys. 
 * Copyright (c) 2008 Mark Boddington (www.badpenguin.co.uk)
 * 
 * This program is licensed under the terms of the GNu GPL version 2.0.
 * The DKIM specification is documented in RFC 4871
 * See: http://www.ietf.org/rfc/rfc4871.txt
 */
package badpenguin.dkim.test;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.util.Calendar;
import java.util.Date;

import org.omg.CORBA_2_3.portable.OutputStream;

import badpenguin.dkim.CanonicalMethod;
import badpenguin.dkim.DkimError;
import badpenguin.dkim.DkimException;
import badpenguin.dkim.DkimSignature;
import badpenguin.dkim.ErrorType;
import badpenguin.dkim.NSKeyStore;
import badpenguin.dkim.Signer;
import badpenguin.dkim.Verifier;

import sun.misc.BASE64Encoder;

public class Test {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		Test test = new Test();
		test.run();
	}
	
	public void run() {
		FileInputStream store = null;
		try {
			store = new FileInputStream("/home/mark/src/dkim_store.dat");
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		KeyManager km = new KeyManager(store, "hello123");
		PrivateKey key = km.getKey("mykey");
		PublicKey cert = km.getCert("mykey");
		
		System.err.println(cert.getFormat());
		
		BASE64Encoder bs = new BASE64Encoder();
		System.err.println("Cert: " + bs.encode(cert.getEncoded()));
		Signer signer = null;
		InputStream msg = null;
		FileOutputStream out = null;
		
		try {
			
			msg = new FileInputStream("/home/mark/savagemail.txt");
			DkimSignature dksig = new DkimSignature("test1", "badpenguin.co.uk");
			dksig.setMethod(CanonicalMethod.RELAXED);
			dksig.addHeader("received");
			dksig.addHeader("received");
			dksig.setItag("dkimadmin@badpenguin.co.uk");
			
			//Long creation = ( new Date().getTime() / 1000 );
			//Long expire = creation + 60;
			
			//dksig.setXtag(expire.toString());
			//dksig.setTtag(creation.toString());
			
			signer = new Signer(dksig,key);
			//dksig.setLtag(50);
			
			signer.setAutoTimeStamp(true);
			signer.setAutoExpire(true);
			signer.setExpireSeconds(12000);
			
			String header = signer.signMail(msg);
			System.out.println(header);
			
			msg.close();
			msg = new FileInputStream("/home/mark/savagemail.txt");
			
			signer.setAutoHeaders(true);
			signer.setAdditionalHeaders(new String[] { "received", "received" });
			
			out = new FileOutputStream("/home/mark/savagesigned.txt");
			signer.signMail(msg, out);
			out.close();
			msg.close();
			
			System.out.println("SLEEPING.....");
			Thread.sleep(2000);
			System.out.println("..... DONE");
			
			NSKeyStore keyStore = new NSKeyStore("dns", "212.23.3.100");
			keyStore.setKeyLimit(5);
			Verifier ver = new Verifier(keyStore, "DKIM");
			ver.setleniency(true);
			msg = new FileInputStream("/home/mark/savagesigned.txt");
			
			//ver.setBadDomains(new String[] { "gmail.com" });
			
			ver.verifyMail(msg);
			System.out.println("Message Verified :-)");
			msg.close();	
			
			//System.exit(1);
			
			ver = new Verifier(keyStore, "DomainKey");
			ver.setleniency(true);
			msg = new FileInputStream("/home/mark/playmail.txt");
			ver.setMaximumSigs(2);
			ver.verifyMail(msg);
			System.out.println("Message Verified :-)");
			msg.close();

		} catch (DkimException d) {
			
			try {msg.close(); } catch (Exception e) { };
			
			if ( d.getErrorType() == ErrorType.NOSIG) {
				// No signature, 
			} else if ( d.getErrorType().equals(ErrorType.TEMPFAIL))  {
				// message failed, but may succeed later
				System.err.println("TEMPFAIL -- " + d.getError().getStatus());
				System.err.println(d.getError().getDescription());
				System.err.println(d.getMessage());
				d.printStackTrace();
			} else if ( d.getErrorType() == ErrorType.PERMFAIL) {
				// message failed and will never verify
				System.err.println("PERMFAIL -- " + d.getError().getStatus());
				System.err.println(d.getError().getDescription());
				System.err.println(d.getMessage());
				d.printStackTrace();
			} else {
				// Internal Library Error
				System.err.println(d.getError().getStatus());
				System.err.println(d.getError().getDescription());
				System.err.println(d.getMessage());
				d.printStackTrace();
			}
		} catch ( Exception e) {
			e.printStackTrace();
		}
		
	}

}
